#####
###  PropertyObject - a generic PropertyObject for ZOPE
##   Copyright (C) 2002	Maik Jablonski (maik.jablonski@uni-bielefeld.de)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

from Globals import Persistent, DTMLFile, InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from urllib import quote
from StripTagParser import StripTags

def manage_addPropertyObject (self, id, title='', REQUEST=None):
	""" Add a new PropertyObject to current ObjectManager """
	
	ob = PropertyObject(id,title,REQUEST)
	self.Destination()._setObject(id,ob)

	# Redirect if requested TTW
	
	if REQUEST:
		try: u=self.DestinationURL()
		except: u=REQUEST['URL1']
		if REQUEST.get('submit')==" Add and Edit ": u="%s/%s" % (u,quote(id))
		REQUEST.RESPONSE.redirect(u+'/manage_main')

manage_addPropertyObjectForm = DTMLFile('dtml/addForm', globals())

class PropertyObject (CatalogAware,SimpleItem,PropertyManager):
	""" ProperyObject: Just a simple PropertyObject with locks """
   
	meta_type='PropertyObject'

	__ac_permissions__=(
	        ('View',('text_content','PrincipiaSearchSource')),
		('Manage properties',('manage_main','manage_propertiesForm','manage_changeProperties','manage_editProperties','manage_addProperties','manage_delProperties')),
		('PropertyObject: Manage',
		('manage_manageForm','manage_PropertyObject')),
		('PropertyObject: Publish',
		('manage_publishForm','manage_publish','manage_unpublish')),
		)
	
	manage_options = (
		{'label': 'Edit',    'action': 'manage_main'},
		{'label': 'View',    'action': ''},
		{'label': 'Publish', 'action': 'manage_publishForm'},
		{'label': 'Manage',  'action': 'manage_manageForm'},
	) + SimpleItem.manage_options
  
	def __init__(self,id,title='',REQUEST=None):
		""" Set id & title of PropertyObject """

		self.id                           = id
		self.title                        = title
		self.object_type                  = ''
		self.property_extensible_schema__ = 1
		self.published			  = 0
		self.PropertyObjectDescription    = ''
				
	manage_main = manage_propertiesForm = DTMLFile('dtml/propertiesForm', globals())

	
	
	manage_manageForm=DTMLFile('dtml/manageForm', globals())
	

	def manage_PropertyObject(self,object_type='', default_catalog='Catalog', property_extensible_schema__=0, PropertyObjectDescription='', update='', REQUEST=None):
		""" Manages a PropertyObject """
		
		old_object_type = self.object_type
		
		self.object_type = object_type
		self.default_catalog=default_catalog
		self.property_extensible_schema__=property_extensible_schema__
		self.PropertyObjectDescription=PropertyObjectDescription
		self.reindex_object()
		
		# update properties
		
		if update:
		
			# search PropertyObjects with same object_type
			# in all subfolders
			
			startFrom = self.aq_parent
					
			resultObjects = startFrom.ZopeFind(
			 startFrom,
			 obj_metatypes=['PropertyObject'],
			 obj_expr="object_type=='%s'" % old_object_type,
			 search_sub=1
			)

			selfMap = self.propertyMap()

			# iterate through the results
			
			for (id,obj) in resultObjects:
			
				# set the manage-vars in the client
				obj.object_type     = self.object_type
				obj.default_catalog = self.default_catalog
				obj.property_extensible_schema__=self.property_extensible_schema__
				obj.PropertyObjectDescription=self.PropertyObjectDescription

				# add new properties to clients
				
				for propertyTuple in selfMap:
					id = propertyTuple['id']
					if not obj.hasProperty(id):
						value = propertyTuple.get('select_variable',self.getProperty(id))
						if not value:
							value=''
						obj.manage_addProperty(id,value,propertyTuple['type'])
				
				# delete old or additional client-properties
				
				clientMap = obj.propertyMap()
				for propertyTuple in clientMap:
					delIds=[]
					id = propertyTuple['id']
					if not self.hasProperty(id):
						delIds.append(id)
					obj.manage_delProperties(delIds)
								
		if REQUEST:
			return self.manage_manageForm(self,REQUEST,manage_tabs_message='Your changes have been saved',update_menu=1)

	manage_publishForm=DTMLFile('dtml/publishForm', globals())

	def manage_publish(self, REQUEST=None):
		""" Change the PropertyObjectView-method """
		
		self.published=1
		self.reindex_object()
		
		if REQUEST:
			return self.manage_publishForm(self,REQUEST,manage_tabs_message="Published PropertyObject",update_menu=1)

	def manage_unpublish(self, REQUEST=None):
		""" Change the PropertyObjectView-method """
		
		self.published=0
		self.reindex_object()
		
		if REQUEST:
			return self.manage_publishForm(self,REQUEST,manage_tabs_message="Unpublished PropertyObject",update_menu=1)


	def manage_addProperty(self, id, value, type, REQUEST=None):
	    """Add property and reindex"""
	    r = PropertyManager.manage_addProperty(self,id,value,type,REQUEST)
	    self.reindex_object()
	    return r
	    
	def manage_delProperties(self, ids=None, REQUEST=None):
	    """Delete properties and reindex"""
	    r = PropertyManager.manage_delProperties(self, ids, REQUEST)
	    self.reindex_object()
	    return r

	def manage_changeProperties(self, REQUEST=None, **kw):
	    """Update properties and reindex"""
	    r = PropertyManager.manage_changeProperties(self, REQUEST, kw=kw)
	    self.reindex_object()
	    return r

	def manage_editProperties(self, REQUEST=None):
	    """Edit Properties and reindex"""
	    r = PropertyManager.manage_editProperties(self, REQUEST)
	    self.reindex_object()
	    return r
	
	def PrincipiaSearchSource(self,REQUEST=None):
		""" Food for the ZCatalog  """
		return StripTags(self.index_html(self.REQUEST))

	def text_content(self,REQUEST=None):
		""" Returns TextContent of PropertyObject """
		
		r=''		
		for value in self.propertyValues():
			r = r+ str(value) + "\n\n"
		return r
	
	def __call__(self, REQUEST={},RESPONSE=None):
		""" Render the page with specified object_type or
		    return the text_content """
				
		r = ''
				
		if hasattr(self,self.object_type):
			r = getattr(self,self.object_type)(self,self.REQUEST,RESPONSE)
		else:
			r = self.text_content(REQUEST)
				
		return r

	index_html=__call__
	
		
InitializeClass(PropertyObject)
